/*
 *  main_unix.cpp - Startup code for Unix
 *
 *  Basilisk II (C) 1997-2001 Christian Bauer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sysdeps.h"

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>

#include <psprtc.h>
#include <pspctrl.h>

#include "cpu_emulation.h"
#include "sys.h"
#include "rom_patches.h"
#include "xpram.h"
#include "timer.h"
#include "video.h"
#include "emul_op.h"
#include "prefs.h"
#include "prefs_editor.h"
#include "macos_util.h"
#include "user_strings.h"
#include "version.h"
#include "main.h"
#include "adb.h"

#define DEBUG 0
#include "debug.h"

// Constants
const int SCRATCH_MEM_SIZE = 0x10000;	// Size of scratch memory area

// CPU and FPU type, addressing mode
int CPUType;
bool CPUIs68060;
int FPUType;
bool TwentyFourBitAddressing;

// Global variables
static uint8 last_xpram[256];		// Buffer for monitoring XPRAM changes
static SceUID OneHzThread = -1;		// 1Hz thread ID.
static SceUID SixtyHzThread = -1;	// 60Hz thread ID.

// Input
SceCtrlData pad;
uint8 LastX = 128, LastY = 128;
bool EnterPressed = false, MouseDown = false;

// Prototypes
int Psp1HzTimer(SceSize, void *);
int Psp60HzTimer(SceSize, void *);

#if USE_SCRATCHMEM_SUBTERFUGE
uint8 *ScratchMem = NULL;			// Scratch memory for Mac ROM writes
#endif

#ifdef ENABLE_MON
static struct sigaction sigint_sa;	// sigaction for SIGINT handler
static void sigint_handler(...);
#endif

/*
 *  Ersatz functions
 */

extern "C" {

#ifndef HAVE_STRDUP
char *strdup(const char *s)
{
	char *n = (char *)malloc(strlen(s) + 1);
	strcpy(n, s);
	return n;
}
#endif

}

/*
 *  Main program
 */
int main(int argc, char **argv)
{
	// Seed random number generator
	pspTime Time;
	sceRtcGetCurrentClockLocalTime(&Time);
	srand(Time.microseconds);

	// Enable the console.
	pspDebugScreenInit();
	pspDebugScreenSetBackColor(0x00ff0000);
	pspDebugScreenClear();

	// Print some info
	printf(GetString(STR_ABOUT_TEXT1), VERSION_MAJOR, VERSION_MINOR);
	printf(" %s\n", GetString(STR_ABOUT_TEXT2));
	
	// Add your name/handle here when you add to this project
	printf("Ported by: ChaosKnight\n\n");

	// Init system routines
	PrefsInit();
	SysInit();

	// Create Memory
#if USE_SCRATCHMEM_SUBTERFUGE
	// Allocate scratch memory
	ScratchMem = (uint8 *)malloc(SCRATCH_MEM_SIZE);
	if (ScratchMem == NULL) {
		ErrorAlert(GetString(STR_NO_MEM_ERR));
		QuitEmulator();
	}
	ScratchMem += SCRATCH_MEM_SIZE/2;	// ScratchMem points to middle of block
#endif
	D(bug("malloc()'ing RAM/ROM... "));
	RAMSize = RAM_AMOUNT & 0xfff00000;	// Round down to 1MB boundary
	ROMSize = 0x00100000;			// Force 1MB of ROM space (largest ROM should be 1MB).

	RAMBaseHost = (uint8*)malloc(RAMSize + ROMSize);
	ROMBaseHost = RAMBaseHost + RAMSize;
	RAMBaseMac = 0;
	ROMBaseMac = RAMSize;
//	InitMEMBaseDiff(RAMBaseHost, RAMBaseMac);

	D(bug("Mac RAM starts at %p (%08x)\n", RAMBaseHost, RAMBaseMac));
	D(bug("Mac ROM starts at %p (%08x)\n", ROMBaseHost, ROMBaseMac));
	
	printf("Loading %s... ", ROM_PATH);
	int rom_fd = sceIoOpen(ROM_PATH, PSP_O_RDONLY, 0777);
	if (rom_fd < 0)
	{
		printf("failed.\n");
		ErrorAlert(GetString(STR_NO_ROM_FILE_ERR));
		QuitEmulator();
	}
	ROMSize = sceIoRead(rom_fd, ((void*)ROMBaseHost), ROMSize);
	printf("success. File size is %d.\n", ROMSize);

	if (ROMSize != 64*1024 && ROMSize != 128*1024 && ROMSize != 256*1024 && ROMSize != 512*1024 && ROMSize != 1024*1024)
	{
		ErrorAlert(GetString(STR_ROM_SIZE_ERR));
		sceIoClose(rom_fd);
		QuitEmulator();
	}
	sceIoClose(rom_fd);

	// Initialize everything
	printf("Initializing... ");
	if (!InitAll())
	{
		printf("failed.\nEmulation halted. Please exit.");
		for (;;);
	}
	printf("success!\n");
	
	// Start the 1Hz interrupt.
//	OneHzThread = sceKernelCreateThread("1hz_thread", Psp1HzTimer, 
//		0x11, 0xFA0, PSP_THREAD_ATTR_USER, 0);
//	if (OneHzThread > -1) sceKernelStartThread(OneHzThread, 0, 0);

	// Start the 60Hz interrupt.
	SixtyHzThread = sceKernelCreateThread("60hz_thread", Psp60HzTimer, 
		0x11, 0xFA0, PSP_THREAD_ATTR_USER, 0);
	if (SixtyHzThread > -1) sceKernelStartThread(SixtyHzThread, 0, 0);
	
	// Configure the controller.
	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	ADBSetRelMouseMode(true);

	// Start 68k and jump to ROM boot routine
	printf("Starting emulation...\n");
	Start680x0();

	QuitEmulator();
	return 0;
}


/*
 *  Quit emulator
 */

void QuitEmulator(void)
{
	D(bug("QuitEmulator\n"));
	
	// Save XPRAM
	SaveXPRAM();

	// Exit 680x0 emulation
	Exit680x0();

	// Stop 1Hz thread
//	sceKernelTerminateDeleteThread(OneHzThread);

	// Stop 60Hz thread
	sceKernelTerminateDeleteThread(SixtyHzThread);

	// Deinitialize everything
	ExitAll();
	
	// Free ROM/RAM areas
	free(RAMBaseHost);
	free(ROMBaseHost);
	
#if USE_SCRATCHMEM_SUBTERFUGE
	// Delete scratch memory area
	if (ScratchMem) {
		free((void *)(ScratchMem - SCRATCH_MEM_SIZE/2));
		ScratchMem = NULL;
	}
#endif

	// Exit system routines
	SysExit();
}

static __attribute__((destructor)) void back_to_kernel (void)
{
	QuitEmulator();
        sceKernelExitGame();
}

/*
 *  SIGINT handler, enters mon
 */

#ifdef ENABLE_MON
static void sigint_handler(...)
{
	uaecptr nextpc;
	extern void m68k_dumpstate(uaecptr *nextpc);
	m68k_dumpstate(&nextpc);
}
#endif


/*
 *  Interrupt flags (must be handled atomically!)
 */

uint32 InterruptFlags = 0;

void SetInterruptFlag(uint32 flag)
{
	InterruptFlags |= flag;		// Pray that this is an atomic operation...
}

void ClearInterruptFlag(uint32 flag)
{
	InterruptFlags &= ~flag;
}

void FlushCodeCache(void *start, uint32 size)
{
}

/*
 *  Delay by specified number of microseconds (<1 second)
 */
void Delay_usec(uint32 usec)
{
	sceKernelDelayThread((SceUInt)usec);
}

int Psp60HzTimer(SceSize args, void *argp)
{
	while (true)
	{
		// Wait another 60Hz.
		sceKernelDelayThread(16625);
		
		// Read input
		sceCtrlReadBufferPositive(&pad, 1);
		
		// Setup analog dead zone and read "mouse"
		if (pad.Lx > 108 && pad.Lx < 148) pad.Lx = LastX = 128;
		else
		{
			if (pad.Lx > 127) ADBMouseMoved( 2, 0);
			if (pad.Lx < 127) ADBMouseMoved(-2, 0);
		}
		if (pad.Ly > 108 && pad.Ly < 148) pad.Ly = LastY = 128;
		else
		{
			if (pad.Ly > 127) ADBMouseMoved(0,  2);
			if (pad.Ly < 127) ADBMouseMoved(0, -2);
		}
		
		if (pad.Buttons != 0)
		{
			if (pad.Buttons & PSP_CTRL_CROSS)
			{
				ADBMouseDown(0);
				MouseDown = true;
			}
			else if ((pad.Buttons & PSP_CTRL_CROSS) && MouseDown)
			{
				ADBMouseUp(0);
				MouseDown = false;
			}
			if (pad.Buttons & PSP_CTRL_START)
			{
				ADBKeyDown(0x24);
				EnterPressed = true;
			}
			else if ((pad.Buttons & PSP_CTRL_START) && EnterPressed)
			{
				ADBKeyUp(0x24);
				EnterPressed = false;
			}
		}
		else
		{
			// Reset anything set.
			if (MouseDown) { ADBMouseUp(0); MouseDown = false; }
			if (EnterPressed) { ADBKeyUp(0x24); EnterPressed = false; }
		}
		
		// Trigger 60Hz interrupt.
		SetInterruptFlag(INTFLAG_60HZ);
		TriggerInterrupt();
	}
}

int Psp1HzTimer(SceSize args, void *argp)
{
	while (true)
	{
		// Wait another 1Hz.
		sceKernelDelayThread(1000);
		
		// Pseudo Mac 1Hz interrupt, update local time
		WriteMacInt32(0x20c, TimerDateTime());

		// Trigger 1Hz interrupt.
//		SetInterruptFlag(INTFLAG_1HZ);
		TriggerInterrupt();
	}
}

/*
 *  Display alert
 */

void ErrorAlert(const char *text)
{
	printf(GetString(STR_SHELL_ERROR_PREFIX), text);
}


/*
 *  Display warning alert
 */

void WarningAlert(const char *text)
{
	printf(GetString(STR_SHELL_WARNING_PREFIX), text);
}


/*
 *  Display choice alert
 */

bool ChoiceAlert(const char *text, const char *pos, const char *neg)
{
	printf(GetString(STR_SHELL_WARNING_PREFIX), text);
	return false;	//!!
}
