/*
 *  sys_psp.cpp - System dependent routines, PlayStation Portable implementation
 *
 *  Basilisk II (C) 1997-2001 Christian Bauer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sysdeps.h"
#include "main.h"
#include "macos_util.h"
#include "prefs.h"
#include "user_strings.h"
#include "sys.h"

#define DEBUG 0
#include "debug.h"

// File handles are pointers to these structures
struct file_handle {
	file_handle *next;	// Pointer to next file handle (must be first in struct!)
	char *name;	// File/device name (copied, for mount menu)
	int fd;				// fd of file/device
	bool is_file;		// Flag: plain file or /dev/something?
	bool read_only;		// Copy of Sys_open() flag
	loff_t start_byte;	// Size of file header (if any)
	loff_t file_size;	// Size of file data (only valid if is_file is true)
};

// Linked list of file handles
static file_handle *first_file_handle = NULL;

/*
 *  Initialization
 */

void SysInit(void)
{
}


/*
 *  Deinitialization
 */

void SysExit(void)
{
}


/*
 *  This gets called when no "floppy" prefs items are found
 *  It scans for available floppy drives and adds appropriate prefs items
 */

void SysAddFloppyPrefs(void)
{
	D(bug("SysAddFloppyPrefs()\n"));
/*
	SceIoDirent dir;
	int dfd = sceIoDopen("ms0:/");
	if (dfd > -1)
	{
		while (sceIoDread(dfd, &dir) > 0)
			if (strstr(dir.d_name, ".dsk") != NULL)
			{
				PrefsAddString("floppy", dir.d_name);
				D(bug("Added floppy: %s\n", dir.d_name));
			}
		sceIoDclose(dfd);
	}
*/
	PrefsAddString("floppy", BOOT_DISK_FILE_NAME);
}


/*
 *  This gets called when no "disk" prefs items are found
 *  It scans for available HFS volumes and adds appropriate prefs items
 */

void SysAddDiskPrefs(void)
{
	D(bug("SysAddDiskPrefs()\n"));
/*
	SceIoDirent dir;
	int dfd = sceIoDopen("ms0:/");
	if (dfd > -1)
	{
		while (sceIoDread(dfd, &dir) > 0)
			if (strstr(dir.d_name, ".hfs") != NULL)
			{
				PrefsAddString("disk", dir.d_name);
				D(bug("Added disk: %s\n", dir.d_name));
			}
		sceIoDclose(dfd);
	}
*/
//	PrefsAddString("disk", "disc0:/");
//	PrefsAddString("disk", "ms0:/");
}


/*
 *  This gets called when no "cdrom" prefs items are found
 *  It scans for available CD-ROM drives and adds appropriate prefs items
 */

void SysAddCDROMPrefs(void)
{
	D(bug("SysAddCDROMPrefs() - UMD is treated as disk, not CDROM.\n"));
}


/*
 *  Add default serial prefs (must be added, even if no ports present)
 */

void SysAddSerialPrefs(void)
{
	D(bug("SysAddSerialPrefs()\n"));
//	PrefsAddString("seriala", "irda0:/");
}


/*
 *  Open file/device, create new file handle (returns NULL on error)
 *
 *  Format for device names: /dev/<name>/<unit>/<open flags>/<start block>/<size (blocks)>/<block size>
 */

void *Sys_open(const char *name, bool read_only)
{
	D(bug("Sys_open(%s, %s)\n", name, read_only ? "read-only" : "read/write"));

	bool isfile = (strstr(name, "/dev/") != name);

	// Open file
	int fd = sceIoOpen(name, ((read_only) ? PSP_O_RDONLY : PSP_O_RDWR), 0777);
	if (fd < 0) return NULL;

	// Create basilisk handle
	file_handle *fh = new file_handle;
	fh->name = strdup(name);
	fh->fd = fd;
	fh->is_file = isfile;
	fh->read_only = read_only;
	fh->start_byte = 0;

	// detect disk layout
	if (fh->is_file)
	{
		loff_t size = sceIoLseek(fd, 0, SEEK_END);
		uint8 data[256];
		sceIoLseek(fd, 0, SEEK_SET);
		sceIoRead(fd, data, 256);
		FileDiskLayout(size, data, fh->start_byte, fh->file_size);
	}

	// Enqueue handle
	fh->next = NULL;
	file_handle *q = first_file_handle;
	if (q) { while (q->next) q=q->next; q->next = fh; }
	else first_file_handle = fh;
	return fh;
}


/*
 *  Close file/device, delete file handle
 */

void Sys_close(void *arg)
{
	D(bug("Sys_close(0x%08x)\n", &arg));
	file_handle *fh = (file_handle *)arg;
	if (!fh)
		return;

	sceIoClose(fh->fd);
	if (fh->name)
		free(fh->name);
	delete fh;
}


/*
 *  Read "length" bytes from file/device, starting at "offset", to "buffer",
 *  returns number of bytes read (or 0)
 */

size_t Sys_read(void *arg, void *buffer, loff_t offset, size_t length)
{
	file_handle *fh = (file_handle *)arg;
	if (!fh) return 0;

	sceIoLseek(fh->fd, offset, SEEK_SET);
	return sceIoRead(fh->fd, buffer, length);
}


/*
 *  Write "length" bytes from "buffer" to file/device, starting at "offset",
 *  returns number of bytes written (or 0)
 */

size_t Sys_write(void *arg, void *buffer, loff_t offset, size_t length)
{
	file_handle *fh = (file_handle *)arg;
	if (!fh) return 0;

	sceIoLseek(fh->fd, offset, SEEK_SET);
	return sceIoWrite(fh->fd, buffer, length);
}


/*
 *  Return size of file/device (minus header)
 */

loff_t SysGetFileSize(void *arg)
{
	file_handle *fh = (file_handle *)arg;
	if (!fh) return true;

	if (fh->is_file) return fh->file_size;
	else return 0;
}


/*
 *  Eject volume (if applicable)
 */

void SysEject(void *arg)
{
}


/*
 *  Format volume (if applicable)
 */

bool SysFormat(void *arg)
{
	return false;
}


/*
 *  Check if file/device is read-only (this includes the read-only flag on Sys_open())
 */

bool SysIsReadOnly(void *arg)
{
	file_handle *fh = (file_handle *)arg;
	if (!fh) return true;

	if (fh->is_file) return fh->read_only;
	else return true;
}


/*
 *  Check if the given file handle refers to a fixed or a removable disk
 */

bool SysIsFixedDisk(void *arg)
{
	return false;
}


/*
 *  Check if a disk is inserted in the drive (always true for files)
 */

bool SysIsDiskInserted(void *arg)
{
	return true;
}


/*
 *  Prevent medium removal (if applicable)
 */

void SysPreventRemoval(void *arg)
{
}


/*
 *  Allow medium removal (if applicable)
 */

void SysAllowRemoval(void *arg)
{
}


/*
 *  Read CD-ROM TOC (binary MSF format, 804 bytes max.)
 */

bool SysCDReadTOC(void *arg, uint8 *toc)
{
	return false;
}


/*
 *  Read CD-ROM position data (Sub-Q Channel, 16 bytes, see SCSI standard)
 */

bool SysCDGetPosition(void *arg, uint8 *pos)
{
	return false;
}


/*
 *  Play CD audio
 */

bool SysCDPlay(void *arg, uint8 start_m, uint8 start_s, uint8 start_f, uint8 end_m, uint8 end_s, uint8 end_f)
{
	return false;
}


/*
 *  Pause CD audio
 */

bool SysCDPause(void *arg)
{
	return false;
}


/*
 *  Resume paused CD audio
 */

bool SysCDResume(void *arg)
{
	return false;
}


/*
 *  Stop CD audio
 */

bool SysCDStop(void *arg, uint8 lead_out_m, uint8 lead_out_s, uint8 lead_out_f)
{
	return false;
}


/*
 *  Perform CD audio fast-forward/fast-reverse operation starting from specified address
 */

bool SysCDScan(void *arg, uint8 start_m, uint8 start_s, uint8 start_f, bool reverse)
{
	return false;
}


/*
 *  Set CD audio volume (0..255 each channel)
 */

void SysCDSetVolume(void *arg, uint8 left, uint8 right)
{
}


/*
 *  Get CD audio volume (0..255 each channel)
 */

void SysCDGetVolume(void *arg, uint8 &left, uint8 &right)
{
	return;
}
